@@ -35,6 +35,7 @@
 #include <libgames-support/games-runtime.h>
 #include <libgames-support/games-sound.h>
 #include <libgames-support/games-stock.h>
+#include <libgames-support/games-fullscreen-action.h>
 
 #include "connect4.h"
 #include "main.h"
@@ -171,7 +172,7 @@ drop_marble (gint r, gint c)
     tile = TILE_PLAYER2;
 
   gboard[r][c] = tile;
-  gfx_draw_tile (r, c, TRUE);
+  gfx_draw_tile (r, c);
 
   column = column_moveto = c;
   row = row_dropto = r;
@@ -190,11 +191,11 @@ drop (void)
     tile = TILE_PLAYER2;
 
   gboard[row][column] = TILE_CLEAR;
-  gfx_draw_tile (row, column, TRUE);
+  gfx_draw_tile (row, column);
 
   row++;
   gboard[row][column] = tile;
-  gfx_draw_tile (row, column, TRUE);
+  gfx_draw_tile (row, column);
 }
 
 
@@ -203,7 +204,7 @@ static void
 move_cursor (gint c)
 {
   gboard[0][column] = TILE_CLEAR;
-  gfx_draw_tile (0, column, TRUE);
+  gfx_draw_tile (0, column);
 
   column = c;
 
@@ -212,7 +213,7 @@ move_cursor (gint c)
   else
     gboard[0][c] = TILE_PLAYER2;
 
-  gfx_draw_tile (0, c, TRUE);
+  gfx_draw_tile (0, c);
 
   column = column_moveto = c;
   row = row_dropto = 0;
@@ -224,7 +225,7 @@ static void
 move (gint c)
 {
   gboard[0][column] = TILE_CLEAR;
-  gfx_draw_tile (0, column, TRUE);
+  gfx_draw_tile (0, column);
 
   column = c;
 
@@ -233,7 +234,7 @@ move (gint c)
   else
     gboard[0][c] = TILE_PLAYER2;
 
-  gfx_draw_tile (0, c, TRUE);
+  gfx_draw_tile (0, c);
 }
 
 
@@ -260,7 +261,7 @@ draw_line (gint r1, gint c1, gint r2, gint c2, gint tile)
   do {
     done = (r1 == r2 && c1 == c2);
     gboard[r1][c1] = tile;
-    gfx_draw_tile (r1, c1, TRUE);
+    gfx_draw_tile (r1, c1);
     if (r1 != r2)
       r1 += d_row;
     if (c1 != c2)
@@ -372,7 +373,6 @@ GtkAction *player_list_action;
 GtkAction *undo_action;
 GtkAction *hint_action;
 GtkAction *fullscreen_action;
-GtkAction *leave_fullscreen_action;
 
 
 static void
@@ -385,36 +385,6 @@ stop_anim (void)
   timeout = 0;
 }
 
-static void
-set_fullscreen_actions (gboolean is_fullscreen)
-{
-  gtk_action_set_sensitive (leave_fullscreen_action, is_fullscreen);
-  gtk_action_set_visible (leave_fullscreen_action, is_fullscreen);
-
-  gtk_action_set_sensitive (fullscreen_action, !is_fullscreen);
-  gtk_action_set_visible (fullscreen_action, !is_fullscreen);
-}
-
-static void
-fullscreen_cb (GtkAction * action)
-{
-  if (action == fullscreen_action) {
-    gtk_window_fullscreen (GTK_WINDOW (app));
-  } else {
-    gtk_window_unfullscreen (GTK_WINDOW (app));
-  }
-}
-
-/* Just in case something else takes us to/from fullscreen. */
-static gboolean
-window_state_cb (GtkWidget * widget, GdkEventWindowState * event)
-{
-  if (event->changed_mask & GDK_WINDOW_STATE_FULLSCREEN)
-    set_fullscreen_actions (event->new_window_state &
-			    GDK_WINDOW_STATE_FULLSCREEN);
-  return FALSE;
-}
-
 static void
 game_init (void)
 {
@@ -671,7 +641,7 @@ on_game_undo (GtkMenuItem * m, gpointer data)
   move_cursor (c);
 
   gboard[r][c] = TILE_CLEAR;
-  gfx_draw_tile (r, c, TRUE);
+  gfx_draw_tile (r, c);
 
   if (get_n_human_players () == 1 && !is_player_human ()) {
     if (moves > 0) {
@@ -683,7 +653,7 @@ on_game_undo (GtkMenuItem * m, gpointer data)
       swap_player ();
       move_cursor (c);
       gboard[r][c] = TILE_CLEAR;
-      gfx_draw_tile (r, c, TRUE);
+      gfx_draw_tile (r, c);
     }
   }
 }
@@ -804,12 +774,11 @@ on_game_scores (GtkMenuItem * m, gpointer data)
 					  GTK_STOCK_CLOSE,
 					  GTK_RESPONSE_CLOSE, NULL);
 
-  gtk_dialog_set_has_separator (GTK_DIALOG (scorebox), FALSE);
   gtk_window_set_resizable (GTK_WINDOW (scorebox), FALSE);
   gtk_container_set_border_width (GTK_CONTAINER (scorebox), 5);
   gtk_box_set_spacing (GTK_BOX (gtk_dialog_get_content_area (GTK_DIALOG (scorebox))), 2);
 
-  g_signal_connect (GTK_OBJECT (scorebox), "destroy",
+  g_signal_connect (scorebox, "destroy",
 		    G_CALLBACK (gtk_widget_destroyed), &scorebox);
 
   vbox = gtk_vbox_new (FALSE, 6);
@@ -1217,9 +1186,9 @@ on_drawarea_resize (GtkWidget * w, GdkEventConfigure * e, gpointer data)
 }
 
 static gboolean
-on_drawarea_expose (GtkWidget * w, GdkEventExpose * e, gpointer data)
+on_drawarea_draw (GtkWidget * w, cairo_t *cr, gpointer data)
 {
-  gfx_expose (&e->area);
+  gfx_expose (cr);
 
   return FALSE;
 }
@@ -1292,10 +1261,6 @@ static const GtkActionEntry action_entry[] = {
   {"Scores", GAMES_STOCK_SCORES, NULL, NULL, NULL,
    G_CALLBACK (on_game_scores)},
   {"Quit", GTK_STOCK_QUIT, NULL, NULL, NULL, G_CALLBACK (on_game_exit)},
-  {"Fullscreen", GAMES_STOCK_FULLSCREEN, NULL, NULL, NULL,
-   G_CALLBACK (fullscreen_cb)},
-  {"LeaveFullscreen", GAMES_STOCK_LEAVE_FULLSCREEN, NULL, NULL, NULL,
-   G_CALLBACK (fullscreen_cb)},
   {"Preferences", GTK_STOCK_PREFERENCES, NULL, NULL, NULL,
    G_CALLBACK (on_settings_preferences)},
   {"Contents", GAMES_STOCK_CONTENTS, NULL, NULL, NULL,
@@ -1321,7 +1286,6 @@ static const char ui_description[] =
   "    </menu>"
   "    <menu action='ViewMenu'>"
   "      <menuitem action='Fullscreen'/>"
-  "      <menuitem action='LeaveFullscreen'/>"
   "    </menu>"
   "    <menu action='SettingsMenu'>"
   "      <menuitem action='Preferences'/>"
@@ -1363,12 +1327,8 @@ create_game_menus (GtkUIManager * ui_manager)
     gtk_action_group_get_action (action_group, "LeaveNetworkGame");
   hint_action = gtk_action_group_get_action (action_group, "Hint");
   undo_action = gtk_action_group_get_action (action_group, "UndoMove");
-  fullscreen_action =
-    gtk_action_group_get_action (action_group, "Fullscreen");
-  leave_fullscreen_action =
-    gtk_action_group_get_action (action_group, "LeaveFullscreen");
-
-  set_fullscreen_actions (FALSE);
+  fullscreen_action = GTK_ACTION (games_fullscreen_action_new ("Fullscreen", GTK_WINDOW (app)));
+  gtk_action_group_add_action_with_accel (action_group, fullscreen_action, NULL);
 }
 
 
@@ -1389,6 +1349,7 @@ create_app (void)
 
   notebook = gtk_notebook_new ();
   gtk_notebook_set_show_tabs (GTK_NOTEBOOK (notebook), FALSE);
+  gtk_notebook_set_show_border (GTK_NOTEBOOK (notebook), FALSE);
 
   g_signal_connect (G_OBJECT (app), "delete_event",
 		    G_CALLBACK (on_game_exit), NULL);
@@ -1398,7 +1359,6 @@ create_app (void)
   statusbar = gtk_statusbar_new ();
   ui_manager = gtk_ui_manager_new ();
 
-  gtk_statusbar_set_has_resize_grip (GTK_STATUSBAR (statusbar), FALSE);
   games_stock_prepare_for_statusbar_tooltips (ui_manager, statusbar);
   create_game_menus (ui_manager);
   menubar = gtk_ui_manager_get_widget (ui_manager, "/MainMenu");
@@ -1432,14 +1392,12 @@ create_app (void)
   gtk_widget_set_events (drawarea, GDK_EXPOSURE_MASK | GDK_BUTTON_PRESS_MASK);
   g_signal_connect (G_OBJECT (drawarea), "configure_event",
 		    G_CALLBACK (on_drawarea_resize), NULL);
-  g_signal_connect (G_OBJECT (drawarea), "expose_event",
-		    G_CALLBACK (on_drawarea_expose), NULL);
+  g_signal_connect (G_OBJECT (drawarea), "draw",
+		    G_CALLBACK (on_drawarea_draw), NULL);
   g_signal_connect (G_OBJECT (drawarea), "button_press_event",
 		    G_CALLBACK (on_button_press), NULL);
   g_signal_connect (G_OBJECT (app), "key_press_event",
 		    G_CALLBACK (on_key_press), NULL);
-  g_signal_connect (G_OBJECT (app), "window_state_event",
-		    G_CALLBACK (window_state_cb), NULL);
 
   /* We do our own double-buffering. */
   gtk_widget_set_double_buffered (GTK_WIDGET (drawarea), FALSE);
@@ -1452,9 +1410,6 @@ create_app (void)
   gtk_widget_hide (chat);
 #endif
 
-  if (!gfx_set_grid_style ())
-    return FALSE;
-
   gfx_refresh_pixmaps ();
   gfx_draw_all ();
 
